/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.system.plugin.assembly;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.UUID;

import org.apache.maven.archiver.MavenArchiveConfiguration;
import org.apache.maven.archiver.MavenArchiver;
import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.ArtifactUtils;
import org.apache.maven.artifact.installer.ArtifactInstaller;
import org.apache.maven.artifact.resolver.filter.ScopeArtifactFilter;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.apache.maven.project.MavenProjectHelper;
import org.bluestemsoftware.open.eoa.system.plugin.assembly.util.FileUtils;
import org.codehaus.plexus.archiver.ArchiverException;
import org.codehaus.plexus.archiver.jar.JarArchiver;

/**
 * Jars a project's dependencies using a runtime scope filter into a single archive and
 * attaches to project using 'jar-with-dependencies' classifier. Packaging depends upon value
 * of 'unpack' parameter. If true, packaging is 'jar', i.e. archive may be added to classpath,
 * if false, it's 'zip', i.e. archive contains jars.
 * 
 * @goal dependencies
 */
public class DependenciesMojo extends AbstractMojo {

    /**
     * The maven project.
     * 
     * @parameter expression="${project}"
     * @required
     * @readonly
     * @description "the maven project to use"
     */
    private MavenProject project;

    /**
     * @parameter expression="${project.artifact}"
     * @required
     * @readonly
     */
    private Artifact artifact;

    /**
     * @parameter expression="${component.org.apache.maven.artifact.installer.ArtifactInstaller}"
     * @required
     * @readonly
     */
    protected ArtifactInstaller installer;

    /**
     * Directory that resources are copied to during the build.
     * 
     * @parameter expression="${project.build.directory}/${project.build.finalName}"
     * @required
     */
    private String workDirectory;

    /**
     * 
     * Project helper injected by maven.
     * 
     * @component
     */
    private MavenProjectHelper projectHelper;

    /**
     * The directory for the generated source attachment.
     * 
     * @parameter expression="${project.build.directory}"
     * @required
     */
    private String outputDirectory;

    /**
     * The name of the attachment to which '-jar-with-dependencies.jar' is appended.
     * 
     * @parameter expression="${project.build.finalName}"
     * @required
     */
    private String finalName;

    /**
     * The Jar archiver.
     * 
     * @parameter expression="${component.org.codehaus.plexus.archiver.Archiver#jar}"
     * @required
     */
    private JarArchiver jarArchiver;

    /**
     * If true dependency artifacts will be unpacked before archiving. Default is false.
     * 
     * @parameter
     * @required
     */
    private boolean unpack;

    /**
     * The maven archiver configuration.
     * @see http://maven.apache.org/shared/maven-archiver/index.html
     * 
     * @parameter
     */
    private MavenArchiveConfiguration archive = new MavenArchiveConfiguration();

    /**
     * A list of versionless keys to exclude from dependencies archive, e.g.
     * groupid:artifactid.
     * 
     * @parameter
     */
    private List<String> excludes = new ArrayList<String>();

    /**
     * Path to custom manifest.
     * 
     * @parameter
     */
    private String manifest;


    private File outputDir;
    private File workDir;
    private File tempDir = new File(System.getProperty("java.io.tmpdir"));

    private static final String[] INCLUDES = new String[] { "**/**" };
    private static final String CLASSIFIER = "jar-with-dependencies";

    /*
     * (non-Javadoc)
     * @see org.apache.maven.plugin.AbstractMojo#execute()
     */
    public void execute() throws MojoExecutionException {

        getLog().info(" ======= DependenciesMojo settings =======");
        getLog().info("workDirectory[" + workDirectory + "]");
        getLog().info("outputDirectory[" + outputDirectory + "]");
        getLog().info("finalName[" + finalName + "]");
        getLog().info("unpack[" + unpack + "]");

        File attachment = null;
        if (unpack) {
            attachment = new File(getOutputDir(), finalName + "-" + CLASSIFIER + ".jar");
        } else {
            attachment = new File(getOutputDir(), finalName + "-" + CLASSIFIER + ".zip");
        }

        MavenArchiver archiver = new MavenArchiver();
        try {
            archiver.setArchiver(jarArchiver);
            archiver.setOutputFile(attachment);
            jarArchiver.addDirectory(getWorkDir());
            if (manifest != null) {
                jarArchiver.setManifest(new File(manifest));
            }
        } catch (Exception ex) {
            throw new MojoExecutionException("Error configuring archiver. ", ex);
        }

        // add the main artifact

        try {
            if (unpack) {
                unpackArchive(artifact.getFile());
            } else {
                jarArchiver.addFile(artifact.getFile(), artifact.getFile().getName());
            }

        } catch (ArchiverException ae) {
            throw new MojoExecutionException(ae.toString());
        }

        // add the dependencies

        ScopeArtifactFilter runtimeFilter = new ScopeArtifactFilter(Artifact.SCOPE_RUNTIME);

        Iterator<?> itr = project.getArtifacts().iterator();
        while (itr.hasNext()) {
            Artifact dependency = (Artifact)itr.next();
            if (runtimeFilter.include(dependency)) {
                if (dependency.getFile() != null) {
                    try {
                        String groupID = dependency.getGroupId();
                        String artifactID = dependency.getArtifactId();
                        String key = ArtifactUtils.versionlessKey(groupID, artifactID);
                        if (!excludes.contains(key)) {
                            getLog().debug("adding dependency " + key);
                            if (unpack) {
                                unpackArchive(dependency.getFile());
                            } else {
                                jarArchiver.addFile(dependency.getFile(), dependency.getFile().getName());
                            }
                        } else {
                            getLog().debug("exluding dependency " + key);
                        }
                    } catch (ArchiverException ae) {
                        throw new MojoExecutionException(ae.toString());
                    }
                }
            }
        }

        try {
            archiver.createArchive(project, archive);
            if (unpack) {
                projectHelper.attachArtifact(project, "jar", CLASSIFIER, attachment);
            } else {
                projectHelper.attachArtifact(project, "zip", CLASSIFIER, attachment);
            }
        } catch (Exception ex) {
            throw new MojoExecutionException("Error assembling source attachment. ", ex);
        }

    }

    protected File getOutputDir() {
        if (outputDir == null) {
            outputDir = new File(outputDirectory);
            if (!outputDir.exists()) {
                outputDir.mkdir();
            }
        }
        return outputDir;
    }

    protected File getWorkDir() {
        if (workDir == null) {
            workDir = new File(workDirectory);
            if (!workDir.exists()) {
                workDir.mkdir();
            }
        }
        return workDir;
    }

    private void unpackArchive(File archive) throws MojoExecutionException {

        // create a temporary directory to which we will unpack the
        // archive and set delete on exit to clean up after ourself

        File target = new File(tempDir, "assembly-plugin-" + UUID.randomUUID().toString());
        target.deleteOnExit();
        target.mkdir();

        FileUtils.unpackArchive(archive, target);

        String[] a = org.codehaus.plexus.util.FileUtils.getDefaultExcludes();
        String[] b = new String[] { "**/META-INF/LICENSE", "**/META-INF/NOTICE", "**/META-INF/MANIFEST.MF" };
        String[] combined = new String[a.length + b.length];
        java.lang.System.arraycopy(b, 0, combined, 0, b.length);
        java.lang.System.arraycopy(a, 0, combined, b.length, a.length);

        try {
            jarArchiver.addDirectory(target, INCLUDES, combined);
        } catch (ArchiverException ae) {
            throw new MojoExecutionException("Error assembling source attachment. ", ae);
        }

    }

}
