/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.system.plugin.assembly.util;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.io.IOUtils;
import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.ArtifactUtils;
import org.apache.maven.artifact.factory.ArtifactFactory;
import org.apache.maven.artifact.metadata.ArtifactMetadataSource;
import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.artifact.resolver.ArtifactCollector;
import org.apache.maven.artifact.resolver.ArtifactResolutionResult;
import org.apache.maven.artifact.resolver.ArtifactResolver;
import org.apache.maven.artifact.resolver.ResolutionNode;
import org.apache.maven.artifact.resolver.filter.AndArtifactFilter;
import org.apache.maven.artifact.resolver.filter.ArtifactFilter;
import org.apache.maven.artifact.resolver.filter.ExcludesArtifactFilter;
import org.apache.maven.artifact.resolver.filter.ScopeArtifactFilter;
import org.apache.maven.artifact.resolver.filter.TypeArtifactFilter;
import org.apache.maven.artifact.versioning.InvalidVersionSpecificationException;
import org.apache.maven.artifact.versioning.VersionRange;
import org.apache.maven.model.Dependency;
import org.apache.maven.model.DependencyManagement;
import org.apache.maven.model.Exclusion;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.logging.Log;
import org.apache.maven.profiles.ProfileManager;
import org.apache.maven.project.InvalidProjectModelException;
import org.apache.maven.project.MavenProject;
import org.apache.maven.project.MavenProjectBuilder;
import static org.bluestemsoftware.open.eoa.system.plugin.assembly.util.DependencyConstants.*;

/**
 * Collects all artifacts from a system dependencies model in accordance with Ext EOA specification
 * maven dependency types.
 */
public final class DependenciesHelper {

    private ArtifactFactory artifactFactory;
    private MavenProjectBuilder projectBuilder;
    private ArtifactMetadataSource metadataSource;
    private ArtifactResolver artifactResolver;
    private ArtifactCollector artifactCollector;
    private AndArtifactFilter artifactFilter;
    private OrArtifactFilter moduleTypeFilter;
    private Artifact dummyArtifact;
    private ArtifactRepository localRepository;
    private ProfileManager profileManager;
    private Set<String> excludes;
    private Log log;

    public DependenciesHelper(ArtifactFactory artifactFactory, MavenProjectBuilder projectBuilder,
            ArtifactMetadataSource metadataSource, ArtifactResolver artifactResolver,
            ArtifactCollector artifactCollector, ArtifactRepository localRepository,
            ProfileManager profileManager, Set<String> excludes, Log log) {

        this.artifactFactory = artifactFactory;
        this.projectBuilder = projectBuilder;
        this.metadataSource = metadataSource;
        this.artifactResolver = artifactResolver;
        this.artifactCollector = artifactCollector;
        this.localRepository = localRepository;
        this.profileManager = profileManager;
        this.excludes = excludes;
        this.log = log;

        ScopeArtifactFilter scopeFilter = new ScopeArtifactFilter(Artifact.SCOPE_RUNTIME);
        TypeArtifactFilter typeFilter = new TypeArtifactFilter("jar");
        artifactFilter = new AndArtifactFilter();
        artifactFilter.add(scopeFilter);
        artifactFilter.add(typeFilter);

        moduleTypeFilter = new OrArtifactFilter();
        moduleTypeFilter.add(new TypeArtifactFilter("eoa-component"));
        moduleTypeFilter.add(new TypeArtifactFilter("eoa-factory"));
        moduleTypeFilter.add(new TypeArtifactFilter("eoa-feature"));

        dummyArtifact = artifactFactory.createArtifact("dummy", "dummy", "dummy", Artifact.SCOPE_RUNTIME, "jar");

    }

    public Set<Artifact> modelSystemDependencies(File projectDescriptor) throws MojoExecutionException {
        
        Set<Artifact> systemDependencies = new HashSet<Artifact>();

        MavenProject project;
        try {
            project = projectBuilder.build(replaceSystemPathVariables(projectDescriptor), localRepository,
                    profileManager);
            project.setDependencyArtifacts(getProjectDependencies(null, project));
        } catch (Exception ex) {
            if (ex instanceof InvalidProjectModelException) {
                throw new MojoExecutionException("Error modeling system dependencies. Invalid system POM. "
                        + ((InvalidProjectModelException)ex).getValidationResult().toString());
            } else {
                throw new MojoExecutionException("Error modeling system dependencies. " + ex);
            }
        }

        // retrieve dependencies declared explicitly within pom, i.e.
        // no transitive dependencies

        Set<?> explicitArtifacts = project.getDependencyArtifacts();

        // iterate over explicit artifacts and add any jar dependencies with
        // system scope to dependency's list of provided dependencies. do not
        // process transitively

        Iterator<?> itr = explicitArtifacts.iterator();
        while (itr.hasNext()) {

            Artifact artifact = (Artifact)itr.next();

            if (!artifact.getScope().equals(MavenProvidedDependency.SCOPE)) {
                continue;
            }
            if (artifact.getType().equals(MavenProvidedDependency.TYPE)) {
                systemDependencies.add(artifact);
            }
        }
        
        // iterate over explicit artifacts and add any jar dependencies with
        // provided scope to dependency's list of shared dependencies. do not
        // process transitively

        itr = explicitArtifacts.iterator();
        while (itr.hasNext()) {

            Artifact artifact = (Artifact)itr.next();

            if (!artifact.getScope().equals(MavenSharedDependency.SCOPE)) {
                continue;
            }
            if (artifact.getType().equals(MavenSharedDependency.TYPE)) {
                if (!isExcluded(artifact)) {
                    systemDependencies.add(artifact);
                }
            }

        }

        // iterate over extension factory dependencies explicitly declared within
        // pom and model each. managed dependencies can specify extension
        // dependencies, i.e. type is 'eoa-factory' and can define feature
        // dependencies, i.e. type is 'eoa-feature'. transitive dependencies
        // are modeled within each returned object. don't worry about circular
        // refs because maven checks this for us

        itr = explicitArtifacts.iterator();
        while (itr.hasNext()) {
            Artifact artifact = (Artifact)itr.next();
            if (!artifact.getScope().equals(MavenFactoryDependency.SCOPE)) {
                if (!artifact.getScope().equals(MavenFeatureDependency.SCOPE)) {
                    continue;
                }
            }
            if (!artifact.getType().equals(MavenFactoryDependency.TYPE)) {
                if (!artifact.getType().equals(MavenFeatureDependency.TYPE)) {
                    continue;
                }
            }
            Set<Artifact> efd = modelFactoryDependency(artifact, project);
            for (Artifact temp : efd) {
                if (!isExcluded(temp)) {
                    systemDependencies.add(temp);
                }
            }
        }

        // model component dependencies - type is eoa-component. component dependencies
        // can define component dependencies, modules dependencies and feature dependencies

        itr = explicitArtifacts.iterator();
        while (itr.hasNext()) {
            Artifact artifact = (Artifact)itr.next();
            if (!artifact.getScope().equals(MavenComponentDependency.SCOPE)) {
                continue;
            }
            if (!artifact.getType().equals(MavenComponentDependency.TYPE)) {
                continue;
            }
            Set<Artifact> cd = modelComponentDependency(artifact, project, new HashSet<String>(), false);
            if (cd != null) {
                for (Artifact temp : cd) {
                    if (!isExcluded(temp)) {
                        systemDependencies.add(temp);
                    }
                }
            }
        }

        return systemDependencies;

    }

    @SuppressWarnings("unchecked")
    private Set<Artifact> modelFactoryDependency(Artifact extensionFactoryArtifact, MavenProject rootProject) throws MojoExecutionException {

        Set<Artifact> factoryDependencies = new HashSet<Artifact>();
        factoryDependencies.add(extensionFactoryArtifact);

        // model factory project and retrieve dependencies declared explicitly within pom, i.e.
        // no transitive dependencies

        MavenProject extensionFactoryProject;
        try {
            extensionFactoryProject = modelDependencyProject(extensionFactoryArtifact, rootProject);
        } catch (Exception ex) {
            throw new MojoExecutionException("Error modeling factory project "
                    + extensionFactoryArtifact.getArtifactId()
                    + ". "
                    + ex);
        }
        Set<Artifact> explicitArtifacts = extensionFactoryProject.getDependencyArtifacts();

        // iterate over explicit artifacts and add any jar dependencies with provided
        // scope not marked as optional to dependency's list of shared dependencies.
        // do not process transitively

        Iterator itr = explicitArtifacts.iterator();
        while (itr.hasNext()) {

            Artifact artifact = (Artifact)itr.next();

            if (!artifact.getScope().equals(MavenSharedDependency.SCOPE)) {
                continue;
            }
            if (artifact.getType().equals(MavenSharedDependency.TYPE)) {
                if (artifact.isOptional()) {
                    continue;
                }
                if (!isExcluded(artifact)) {
                    factoryDependencies.add(artifact);
                }
            }

        }

        // iterate over explicit artifacts and add any jar dependencies with
        // scope 'compile' and optional equals true to list of scoped artifacts

        Set<Artifact> scopedArtifacts = new HashSet<Artifact>();
        itr = explicitArtifacts.iterator();
        while (itr.hasNext()) {
            Artifact artifact = (Artifact)itr.next();
            if (!artifact.getScope().equals(MavenScopedDependency.SCOPE)) {
                continue;
            }
            if (artifact.getType().equals(MavenScopedDependency.TYPE)) {
                if (!artifact.isOptional()) {
                    continue;
                }
                scopedArtifacts.add(artifact);
            }
        }
        
        // collect transitive dependencies defined by scoped artifacts, i.e.
        // artifacts with scope compile or runtime and not marked as optional

        ArtifactResolutionResult arr = null;
        try {
            Map<String, Artifact> managedVersionMap = createManagedVersionMap(extensionFactoryProject);
            arr = artifactCollector.collect(scopedArtifacts, dummyArtifact, managedVersionMap, localRepository,
                    extensionFactoryProject.getRemoteArtifactRepositories(), metadataSource, artifactFilter,
                    new ArrayList());
        } catch (Exception ex) {
            throw new MojoExecutionException("Error resolving transitive dependencies. " + ex);
        }

        // iterate over each dependency node and resolve the artifact which may
        // involve retrieving from a remote repo, i.e. log any errors

        Set<Artifact> resolvedArtifacts = new HashSet<Artifact>();
        itr = arr.getArtifactResolutionNodes().iterator();
        while (itr.hasNext()) {
            ResolutionNode node = (ResolutionNode)itr.next();
            try {
                Artifact ra = node.getArtifact();
                resolveArtifact(ra, node.getRemoteRepositories());
                resolvedArtifacts.add(ra);
            } catch (Exception ex) {
                throw new MojoExecutionException("error resolving "
                        + node.getArtifact().toString()
                        + ". "
                        + ex.getMessage());
            }
        }

        // now iterate over scoped artifacts and all transitive artifacts
        // and create a scoped dependency object for each

        for (Artifact ra : resolvedArtifacts) {
            if (!isExcluded(ra)) {
                factoryDependencies.add(ra);
            }
        }

        // iterate over feature factory dependencies, i.e. any artifact with type
        // eoa-feature and scope runtime and process them recursively. extension
        // factories are only allowed to define feature dependencies, i.e.
        // dependencies on non-feature extension factory types are not allowed

        itr = explicitArtifacts.iterator();
        while (itr.hasNext()) {
            Artifact artifact = (Artifact)itr.next();
            if (!artifact.getScope().equals(MavenFeatureDependency.SCOPE)) {
                continue;
            }
            if (!artifact.getType().equals(MavenFeatureDependency.TYPE)) {
                continue;
            }
            Set<Artifact> ffd = modelFactoryDependency(artifact, rootProject);
            for (Artifact temp : ffd) {
                if (!isExcluded(temp)) {
                    factoryDependencies.add(temp);
                }
            }            
        }

        return factoryDependencies;

    }

    private MavenProject modelDependencyProject(Artifact factoryArtifact, MavenProject parentProject) throws Exception {

        String ref = factoryArtifact.getGroupId()
                + "/"
                + factoryArtifact.getArtifactId()
                + "/"
                + factoryArtifact.getVersion()
                + "/"
                + factoryArtifact.getType();

        log.debug("modeling dependency project " + ref);

        MavenProject dependencyProject = null;
        try {
            Artifact pomArtifact = artifactFactory.createProjectArtifact(factoryArtifact.getGroupId(),
                    factoryArtifact.getArtifactId(), factoryArtifact.getVersion());
            artifactResolver.resolve(pomArtifact, parentProject.getRemoteArtifactRepositories(), localRepository);
            File temp = pomArtifact.getFile();
            pomArtifact.setFile(replaceSystemPathVariables(temp));
            dependencyProject = projectBuilder.buildFromRepository(pomArtifact, parentProject
                    .getRemoteArtifactRepositories(), localRepository);
            dependencyProject.setDependencyArtifacts(getProjectDependencies(factoryArtifact, dependencyProject));
        } catch (org.apache.maven.project.InvalidProjectModelException ie) {
            throw new MojoExecutionException("Error validating POM at location '"
                    + ie.getPomLocation()
                    + "'. "
                    + ie.getValidationResult());
        }

        return dependencyProject;

    }

    /*
     * this is a workaround for what i think is a bug in maven project builder, i.e. env vars
     * in systempath value (scope=system) are not resolved before validating project model (as
     * of version 2.1 snapshot).
     */
    private File replaceSystemPathVariables(File file) throws Exception {
        String localRepoPath = localRepository.getBasedir();
        File parentDir = new File(localRepoPath).getParentFile();
        String pathRoot = parentDir.toURI().getSchemeSpecificPart();
        if (pathRoot.endsWith("/")) {
            pathRoot = pathRoot.substring(0, pathRoot.length() - 1);
        }
        String pom = IOUtils.toString(new FileInputStream(file), "UTF-8");
        pom = pom.replace("${env.ALAKAI_HOME}", pathRoot);
        File temp = File.createTempFile("tmp", ".pom");
        temp.deleteOnExit();
        FileOutputStream out = new FileOutputStream(temp);
        ByteArrayInputStream in = new ByteArrayInputStream(pom.getBytes("UTF-8"));
        IOUtils.copy(in, out);
        out.flush();
        out.close();
        return temp;
    }

    @SuppressWarnings("unchecked")
    private Set<Artifact> modelComponentDependency(Artifact componentArtifact, MavenProject rootProject, Set<String> processed, boolean isTransitive) throws MojoExecutionException {

        Set<Artifact> componentDependencies = new HashSet<Artifact>();
        componentDependencies.add(componentArtifact);

        // if we've already processed the component dependency somewhere
        // up the hierarchy, we quietly ignore it. circular refs are
        // allowed, i.e. two application deployments may depend on
        // one another at runtime, i.e. they may invoke one another. note
        // that although we are creating a hierarchical model here, it
        // effectively ends up being flattened into a set of dependencies
        // for which the associated component deployment must be deployed

        String ref = componentArtifact.getGroupId()
                + "/"
                + componentArtifact.getArtifactId()
                + "/"
                + componentArtifact.getVersion()
                + "/"
                + componentArtifact.getType();

        if (processed.contains(ref)) {
            return null;
        }

        // model component project and retrieve dependencies declared explicitly within
        // pom, i.e. no transitive dependencies

        MavenProject componentProject;
        try {
            componentProject = modelDependencyProject(componentArtifact, rootProject);
        } catch (Exception ex) {
            throw new MojoExecutionException("Error modeling component project " + ref + ". " + ex);
        }

        Set<Artifact> explicitArtifacts = componentProject.getDependencyArtifacts();

        // iterate over explicit artifacts and add any jar dependencies with provided
        // scope not marked as optional to dependency's list of shared dependencies.
        // do not process transitively. note that if we're processing a transitive
        // component dependency we do not process classpath artifacts

        if (!isTransitive) {
            Iterator<?> itr = explicitArtifacts.iterator();
            while (itr.hasNext()) {

                Artifact artifact = (Artifact)itr.next();

                if (!artifact.getScope().equals(MavenSharedDependency.SCOPE)) {
                    continue;
                }
                if (artifact.getType().equals(MavenSharedDependency.TYPE)) {
                    if (artifact.isOptional()) {
                        continue;
                    }
                    if (!isExcluded(artifact)) {
                        componentDependencies.add(artifact);
                    }
                }

            }
        }

        // iterate over explicit artifacts and add any jar dependencies with
        // scope 'compile' and optional equals true to list of scoped artifacts
        // note that if we're processing a transitive component dependency we
        // do not process classpath artifacts

        if (!isTransitive) {

            Set<Artifact> scopedArtifacts = new HashSet<Artifact>();

            Iterator<?> itr = explicitArtifacts.iterator();
            while (itr.hasNext()) {
                Artifact artifact = (Artifact)itr.next();
                if (!artifact.getScope().equals(MavenScopedDependency.SCOPE)) {
                    continue;
                }
                if (artifact.getType().equals(MavenScopedDependency.TYPE)) {
                    if (!artifact.isOptional()) {
                        continue;
                    }
                    scopedArtifacts.add(artifact);
                }
            }

            // collect transitive dependencies defined by scoped artifacts, i.e.
            // artifacts with scope compile or runtime and not marked as optional

            ArtifactResolutionResult arr;
            try {
                Map<String, Artifact> managedVersionMap = createManagedVersionMap(componentProject);
                arr = artifactCollector.collect(scopedArtifacts, dummyArtifact, managedVersionMap,
                        localRepository, componentProject.getRemoteArtifactRepositories(), metadataSource,
                        artifactFilter, new ArrayList());
            } catch (Exception ex) {
                throw new MojoExecutionException("Error resolving transitive dependencies. " + ex);
            }

            // iterate over each dependency node and resolve the artifact which may
            // involve retrieving from a remote repo, i.e. log any errors

            Set<Artifact> resolvedArtifacts = new HashSet<Artifact>();
            itr = arr.getArtifactResolutionNodes().iterator();
            while (itr.hasNext()) {
                ResolutionNode node = (ResolutionNode)itr.next();
                try {
                    Artifact ra = node.getArtifact();
                    resolveArtifact(ra, node.getRemoteRepositories());
                    resolvedArtifacts.add(ra);
                } catch (Exception ex) {
                    throw new MojoExecutionException("error resolving "
                            + node.getArtifact().toString()
                            + ". "
                            + ex.getMessage());
                }
            }

            // now iterate over scoped artifacts and all transitive artifacts
            // and create a scoped dependency object for each

            for (Artifact ra : resolvedArtifacts) {
                if (!isExcluded(ra)) {
                    componentDependencies.add(ra);
                }
            }

        }

        // iterate over component dependencies, i.e. any artifact with type
        // eoa-component and scope runtime. note that if this dependency is
        // marked as optional, we do not process transitive component
        // dependencies

        if (!componentArtifact.isOptional()) {

            Iterator<?> itr = explicitArtifacts.iterator();
            while (itr.hasNext()) {

                Artifact artifact = (Artifact)itr.next();
                if (!artifact.getScope().equals(MavenComponentDependency.SCOPE)) {
                    continue;
                }
                if (!artifact.getType().equals(MavenComponentDependency.TYPE)) {
                    continue;
                }

                Set<Artifact> temp = modelComponentDependency(artifact, rootProject, processed, true);
                if (temp != null) {
                    for (Artifact t : temp) {
                        if (!isExcluded(t)) {
                            componentDependencies.add(t);
                        }
                    }
                }
            }

        }

        // iterate over feature dependencies, i.e. any artifact with type
        // eoa-feature and scope runtime. note that if we're processing a transitive
        // component dependency we do not process feature dependencies, i.e.
        // runtime engine defs are only loaded if defined within a root
        // component dependency. no engine - no required features (features
        // required by runtime bindings are defined within binding factory
        // deployment which must be declared within root of system pom)

        if (!isTransitive) {
            Iterator<?> itr = explicitArtifacts.iterator();
            while (itr.hasNext()) {
                Artifact artifact = (Artifact)itr.next();
                if (!artifact.getScope().equals(MavenFeatureDependency.SCOPE)) {
                    continue;
                }
                if (!artifact.getType().equals(MavenFeatureDependency.TYPE)) {
                    continue;
                }
                Set<Artifact> ffd = modelFactoryDependency(artifact, rootProject);
                for (Artifact temp : ffd) {
                    if (!isExcluded(temp)) {
                        componentDependencies.add(temp);
                    }
                }    
            }
        }

        // iterate over explicit artifacts and add any dependency other than a
        // dependency with type 'jar', 'eoa-component', or 'eoa-feature' to
        // dependency's list of module dependencies. do not process transitively.
        // note that if we're processing a transitive component dependency we do
        // not process module dependencies, i.e. runtime engine defs are only
        // loaded if defined within a root component dependency. no engine - no
        // modules to deploy

        if (!isTransitive) {
            Iterator<?> itr = explicitArtifacts.iterator();
            while (itr.hasNext()) {
                Artifact artifact = (Artifact)itr.next();
                if (!moduleTypeFilter.include(artifact) && artifact.getScope().equals(MavenModuleDependency.SCOPE)) {
                    if (!isExcluded(artifact)) {
                        componentDependencies.add(artifact);
                    }
                }
            }
        }

        processed.add(ref);

        return componentDependencies;

    }

    private Set<Artifact> getProjectDependencies(Artifact dependencyArtifact, MavenProject dependencyProject) throws Exception {

        Set<Artifact> projectDependencies = new HashSet<Artifact>();

        for (Object dependencyObj : dependencyProject.getDependencies()) {

            Dependency dependency = (Dependency)dependencyObj;
            String groupID = dependency.getGroupId();
            String artifactID = dependency.getArtifactId();
            VersionRange versionRange = VersionRange.createFromVersionSpec(dependency.getVersion());
            String scope = dependency.getScope() == null ? Artifact.SCOPE_COMPILE : dependency.getScope();
            String type = dependency.getType() == null ? "jar" : dependency.getType();
            String classifier = dependency.getClassifier();
            boolean isOptional = dependency.isOptional();

            Artifact artifact = artifactFactory.createDependencyArtifact(groupID, artifactID, versionRange, type,
                    classifier, scope, isOptional);
            projectDependencies.add(artifact);

            if (scope.equals(Artifact.SCOPE_SYSTEM)) {
                File file = new File(dependency.getSystemPath());
                artifact.setFile(file);
                continue;
            }

            if (dependency.getExclusions() != null) {
                List<String> patterns = new ArrayList<String>();
                for (Object exclusionObj : dependency.getExclusions()) {
                    Exclusion exclusion = (Exclusion)exclusionObj;
                    StringBuilder sb = new StringBuilder();
                    sb.append(exclusion.getGroupId());
                    sb.append(":");
                    sb.append(exclusion.getArtifactId());
                    patterns.add(sb.toString());
                }
                ExcludesArtifactFilter excludesFilter = new ExcludesArtifactFilter(patterns);
                artifact.setDependencyFilter(excludesFilter);
            }

            resolveArtifact(artifact, dependencyProject.getRemoteArtifactRepositories());

        }

        return projectDependencies;

    }

    private void resolveArtifact(Artifact artifact, List<?> remoteRepositories) throws Exception {

        // if artifact is a snapshot, save off the artifact version before resolving. for
        // some reason, as of version 2.1-SNAPSHOT, the DefaultArtifactResolver overwrites
        // SNAPSHOT with timestamp version format (baseVersion and version), which will
        // create a VERY obscure bug, i.e. pom will not be located and will instead be
        // stubbed, i.e. no dependencies ...

        if (artifact.isSnapshot()) {
            String version = artifact.getVersion();
            artifactResolver.resolve(artifact, remoteRepositories, localRepository);
            artifact.selectVersion(version);
        } else {
            artifactResolver.resolve(artifact, remoteRepositories, localRepository);
        }

    }

    @SuppressWarnings("unchecked")
    private Map<String, Artifact> createManagedVersionMap(MavenProject project) throws InvalidVersionSpecificationException {
        DependencyManagement dependencyManagement = project.getDependencyManagement();
        if (dependencyManagement == null) {
            return Collections.EMPTY_MAP;
        }
        if (dependencyManagement.getDependencies() == null) {
            return Collections.EMPTY_MAP;
        }
        Map<String, Artifact> managedVersionMap = new HashMap<String, Artifact>();
        Iterator<?> itr = dependencyManagement.getDependencies().iterator();
        while (itr.hasNext()) {
            Dependency d = (Dependency)itr.next();
            VersionRange versionRange = VersionRange.createFromVersionSpec(d.getVersion());
            Artifact artifact = artifactFactory.createDependencyArtifact(d.getGroupId(), d.getArtifactId(),
                    versionRange, d.getType(), d.getClassifier(), d.getScope(), d.isOptional());
            managedVersionMap.put(d.getManagementKey(), artifact);
        }
        return managedVersionMap;
    }

    static class OrArtifactFilter implements ArtifactFilter {

        private final List<ArtifactFilter> filters = new ArrayList<ArtifactFilter>();

        public boolean include(Artifact artifact) {
            boolean include = false;
            for (ArtifactFilter filter : filters) {
                if (filter.include(artifact)) {
                    include = true;
                    break;
                }
            }
            return include;
        }

        public void add(ArtifactFilter artifactFilter) {
            filters.add(artifactFilter);
        }

    }

    private boolean isExcluded(Artifact artifact) {
        String groupID = artifact.getGroupId();
        String artifactID = artifact.getArtifactId();
        String key = ArtifactUtils.versionlessKey(groupID, artifactID);
        return excludes.contains(key);
    }

}
